# Serene Bach plugin for handling MT flash calendar.
#
# Release 1.00 (September 13,2006)
#         1.01 (September 21,2006) correspond to euc.
#         1.02 (September 26,2006) use sb::Language.
#         1.03 (September 28,2006) add path of calendar.xml.
#         2.00 (April 10,2007) correspond to Flash Calendar 3.00.
#         2.01 (April 18,2007) fix bug for euc-jp.
#         2.02 (May 28,2007) fix bug for euc-jp and fix typo.
#         2.03 (May 28,2007) add CDATA and fix timezone display.
#         2.04 (Jun 4,2007) fix bug for channel element.
#
# This software is provided as-is. You may use it for commercial or 
# personal use. If you distribute it, please keep this notice intact.
#
# Copyright (c) 2005-2007 Yujiro Araki

package sb::Plugin::FlashCalendarHandler;

use sb::Plugin();

our @linklist;
our $oldfilename = '';

sb::Plugin->register_plugin(
    'lang' => {
        'ja' => 'euc',
        'en' => 'ascii',
    },
    'text' => {
        'type'    => 'cms',
        'name'    => 'FlashCalendarHandler',
        'text'    => 'Handle MT Flash Calendar',
        'author'  => 'Yujiro Araki',
        'detail'  => 'http://www.koikikukan.com/',
        'version' => '2.03',
    },
    'data' => 0,
    'file' => 'flashcalendarhandler.txt',
);

# register callback function to cms entry point
sb::Plugin->register_content_module(
    'type'     => 'main',
    'callback' => \&sb::Content::FlashCalendarHandler::_list_all,
    'field'    => 'flash_calendar',
);

package sb::Content::FlashCalendarHandler;

use sb::Language ();

sub _output {
    my $filename = shift;
    $filename .= '.xml';
    my $body = join("\n",@linklist);
    $filename = sb::Config->get->value('conf_dir_base') . 'xml/' .$filename;
    eval {
        open(TOPOUT,">$filename") or die(sb::Language->get->string('error_file_open') . $filename);
        binmode(TOPOUT);
        print TOPOUT $body;
        close(TOPOUT);
        chmod(sb::Config->get->value('basic_file_attr'), $filename);
    };
}

sub _xml_entitize {
    my $text = shift;
    $text =~ s/&/&amp;/g;
    $text =~ s/</&lt;/g;
    $text =~ s/>/&gt;/g;
    return($text);
}

sub _list_all {
    my $cms = shift;
    my %var = @_;

    return 0 if $var{'mode'} ne 'page';

    my $dir = sb::Config->get->value('conf_dir_base') . 'xml/';
    if (!-e $dir) {
        eval{ umask(0) };
        mkdir('xml', sb::Config->get->value('basic_dir_attr'));
    }

    my $lang = sb::Config->get->value('conf_lang');
    my $charset = $var{'lang'}->charset;
    push(@linklist,qq|<?xml version="1.0" encoding="UTF-8"?>\n<rdf:RDF\nxmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"\nxmlns:dc="http://purl.org/dc/elements/1.1/"\nxmlns:sy="http://purl.org/rss/1.0/modules/syndication/"\nxmlns:admin="http://webns.net/mvcb/"\nxmlns:cc="http://web.resource.org/cc/"\nxmlns="http://purl.org/rss/1.0/">|);

    my $siteurl = $var{'conf'}->value('conf_srv_base');
    push(@linklist,qq|<channel rdf:about="$siteurl">\n</channel>|);

    my @entries = sb::Data->load('Entry','cond'=>{'stat'=>[1]},'sort'=>'date','order'=>1,'detail' => 'on');
    for my $entry (@entries) {
        my $url = $entry->permalink('cat'=>$var{'cat'},
                    'type'=>($var{'mode'} eq 'mob') ? 'Mobile' : $var{'conf'}->value('conf_entry_archive')
        );
        my $date = sb::Time->format(
            'time' => $entry->date,
            'form' => '%Year%-%Mon%-%Day%T%Hour%:%Min%:%Sec%' . substr($var{'conf'}->value('conf_timezone'),0,3) . ':' . substr($var{'conf'}->value('conf_timezone'),3,2),
            'zone' => $var{'conf'}->value('conf_timezone')
        );
        my $filename = sb::Time->format(
            'time' => $entry->date,
            'form' => '%Year%_%Mon%',
            'zone' => $var{'conf'}->value('conf_timezone')
        );
        my $dailyfilename = sb::Time->format(
            'time' => $entry->date,
            'form' => '%Year%%Mon%%Day%',
            'zone' => $var{'conf'}->value('conf_timezone')
        );

        if (($oldfilename ne '') && $filename ne $oldfilename) {
            push(@linklist,qq|</rdf:RDF>\n|);
            &_output($oldfilename);
            @linklist = ();
            push(@linklist,qq|<?xml version="1.0" encoding="UTF-8"?>\n<rdf:RDF\nxmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"\nxmlns:dc="http://purl.org/dc/elements/1.1/"\nxmlns:sy="http://purl.org/rss/1.0/modules/syndication/"\nxmlns:admin="http://webns.net/mvcb/"\nxmlns:cc="http://web.resource.org/cc/"\nxmlns="http://purl.org/rss/1.0/">|);
            push(@linklist,qq|<channel rdf:about="$siteurl">\n</channel>|);
        }
        $oldfilename = $filename;

        $title = _xml_entitize($entry->subj);
        if ($charset eq 'EUC-JP') {
            $title = sb::Language->get->convert($title, 'utf8');
        }
        my $sum = ($entry->sum ne '') ? $entry->sum : $entry->formated_body;
        if ($charset eq 'EUC-JP') {
            $sum = sb::Language->get->convert($sum, 'utf8');
        }
        push(@linklist,qq|<item rdf:about="$url">\n<title>$title</title>\n<link_daily>${siteurl}sb.cgi?day=$dailyfilename</link_daily>\n<link_Individual>$url</link_Individual>\n<description>$sum</description>|);

        if ($entry->body ne '') {
            my $body = _xml_entitize($entry->body);
            if ($charset eq 'EUC-JP') {
                $body = sb::Language->get->convert($body, 'utf8');
            }
            push(@linklist,qq|<content type="text/html" mode="escaped" xml:lang="ja">\n<![CDATA[\n$body\n]]>\n</content>|);
        }
        if ($entry->more ne '') {
            my $more = _xml_entitize($entry->more);
            if ($charset eq 'EUC-JP') {
                $more = sb::Language->get->convert($more, 'utf8');
            }
            push(@linklist,qq|<content type="text/html" mode="escaped" xml:lang="ja">\n<![CDATA[\n$more\n]]>\n</content>|);
        }
        push(@linklist,qq|<dc:date>$date</dc:date>\n</item>|);
    }
    push(@linklist,qq|</rdf:RDF>\n|);
    &_output($oldfilename);

    return 0 if($@);
    return 1;
}

1;
__END__
